/***********************************************************************************************************************
 * Copyright (C) 2010-2013 by the Stratosphere project (http://stratosphere.eu)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 **********************************************************************************************************************/

package eu.stratosphere.nephele.io.channels.serialization;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Random;

/**
 */
public class UTFStringType implements SerializationTestType
{
	private static final int MAX_LEN = 1500;
	
	private String value;
	

	public UTFStringType()
	{
		this.value = "";
	}
	
	private UTFStringType(String value)
	{
		this.value = value;
	}
	

	@Override
	public UTFStringType getRandom(Random rnd)
	{
		final StringBuilder bld = new StringBuilder();
		final int len = rnd.nextInt(MAX_LEN + 1);
		
		for (int i = 0; i < len; i++) {
			bld.append((char) rnd.nextInt(Character.MAX_VALUE));
		}
		
		return new UTFStringType(bld.toString());
	}
	

	@Override
	public void write(DataOutput out) throws IOException
	{
		out.writeUTF(this.value);
	}


	@Override
	public void read(DataInput in) throws IOException
	{
		this.value = in.readUTF();
	}


	@Override
	public int hashCode()
	{
		return this.value.hashCode();
	}


	@Override
	public boolean equals(Object obj)
	{
		if (obj instanceof UTFStringType) {
			UTFStringType other = (UTFStringType) obj;
			return this.value.equals(other.value);
		} else {
			return false;
		}
	}
}
